;******************************************************************************
;
; Copyright(C) 2015 NXP Semiconductors N.V.
;
; All rights reserved.
;
;******************************************************************************
; * @file     keil_startup_qn908x.s
; * @brief    CMSIS Cortex-M4 Core Device Startup File for
; *           Device QN908x
; * @version  V3.01
; * @date     06. March 2012
; *
; * @note
; * Copyright (C) 2012 ARM Limited. All rights reserved.
; *
; * @par
; * ARM Limited (ARM) is supplying this software for use with Cortex-M
; * processor based microcontrollers.  This file can be freely distributed
; * within development tools that are supporting such ARM based processors.
; *
; * @par
; * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
; * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
; * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
; * ARM SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR
; * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
; *
; ******************************************************************************/
;/*
;//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
;*/
#include "system_config.h"

SYS_MODE_REG                EQU     0x40000014
REMAP_BIT                   EQU     0x00000001

; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000800

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp


; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000000

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base     
Heap_Mem        SPACE   Heap_Size
__heap_limit


                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset

                AREA    RESET, DATA, READONLY
                EXPORT  __Vectors
                EXPORT  __Vectors_End
                EXPORT  __Vectors_Size

__Vectors       DCD     __initial_sp              ; Top of Stack
                DCD     Reset_Handler             ; Reset Handler
                DCD     NMI_Handler               ; NMI Handler
                DCD     HardFault_Handler         ; Hard Fault Handler
                DCD     MemManage_Handler         ; MPU Fault Handler
                DCD     BusFault_Handler          ; Bus Fault Handler
                DCD     UsageFault_Handler        ; Usage Fault Handler
                DCD     0                         ; Reserved
                DCD     0                         ; Reserved
                DCD     0x00000100                ; Application version: 0.0.1.0
                DCD     0                         ; Reserved
                DCD     SVC_Handler               ; SVCall Handler
                DCD     DebugMon_Handler          ; Debug Monitor Handler
                DCD     0                         ; Reserved
                DCD     PendSV_Handler            ; PendSV Handler
                DCD     SysTick_Handler           ; SysTick Handler

                ; External Interrupts
; ToDo:  Add here the vectors for the device specific external interrupts handler
                DCD     DIG_WAKEUP_IRQHandler     ;  0:  DIG wakeup
                DCD     OSC_EN_IRQHandler         ;  1:  BLE IP OSC_EN
                DCD     ACMP0_IRQHandler          ;  2:  ACMP0
                DCD     ACMP1_IRQHandler          ;  3:  ACMP1
                DCD     CHARGER_IRQHandler        ;  4:  CHARGER
                DCD     RTC_IRQHandler            ;  5:  RTC
                DCD     RTC_NEW_IRQHandler        ;  6:  RTC NEW
                DCD     TSC_WAKEUP_IRQHandler     ;  7:  TCS wakeup
                DCD     TSC_IRQHandler            ;  8:  TCS
                DCD     GPIOA_IRQHandler          ;  9:  GPIOA
                DCD     GPIOB_IRQHandler          ; 10:  GPIOB
                DCD     DMA_IRQHandler            ; 11:  DMA
                DCD     PIN_INTR0_IRQHandler      ; 12:  PINT0
                DCD     PIN_INTR1_IRQHandler      ; 13:  PINT1
                DCD     PIN_INTR2_IRQHandler      ; 14:  PINT2
                DCD     PIN_INTR3_IRQHandler      ; 15:  PINT3
                DCD     OSC_INT_LOW_IRQHandler    ; 16:  OSC_INT_LOW
                DCD     USB_IRQHandler            ; 17:  USB
                DCD     FLEXCOMM0_IRQHandler      ; 18:  FLEXCOMM0
                DCD     FLEXCOMM1_IRQHandler      ; 19:  FLEXCOMM1
                DCD     FLEXCOMM2_IRQHandler      ; 20:  FLEXCOMM2
                DCD     FLEXCOMM3_IRQHandler      ; 21:  FLEXCOMM3
                DCD     BLE_IRQHandler            ; 22:  BLE
                DCD     FSP_IRQHandler            ; 23:  FSP
                DCD     QDEC0_IRQHandler          ; 24:  QDEC0
                DCD     QDEC1_IRQHandler          ; 25:  QDEC1
                DCD     TIMER0_IRQHandler         ; 26:  Timer 0
                DCD     TIMER1_IRQHandler         ; 27:  Timer 1
                DCD     TIMER2_IRQHandler         ; 28:  Timer 2
                DCD     TIMER3_IRQHandler         ; 29:  Timer 3
                DCD     WDT_IRQHandler            ; 30:  Watchdog Timer
                DCD     ADC_IRQHandler            ; 31:  ADC
                DCD     DAC_IRQHandler            ; 32:  DAC
                DCD     XTAL_IRQHandler           ; 33:  XTAL ready
                DCD     FLASH_IRQHandler          ; 34:  FLASH
                DCD     QUAD_SPI_IRQHandler       ; 35:  QUAD_SPI
                DCD     PWM_IRQHandler            ; 36:  PWM
                DCD     RTC_CAP_IRQHandler        ; 37:  RTC capture
                DCD     RNG_IRQHandler            ; 38:  RNG
                DCD     CHRG_I2C_IRQHandler       ; 39:  Charger I2c
                DCD     CALIB_IRQHandler          ; 40:  Calibration
                DCD     USB_FIQ_IRQHandler        ; 41:  USB FIQ
                DCD     BLE_TX_IRQHandler         ; 42:  BLE Tx
                DCD     BLE_RX_IRQHandler         ; 43:  BLE Rx
                DCD     BLE_FREQ_HOP_IRQHandler   ; 44:  BLE FREQ HOP
                DCD     PROP_TX_IRQHandler        ; 45:  Proprietary tx
                DCD     PROP_RX_IRQHandler        ; 46:  Proprietary rx
                DCD     DP_IRQHandler             ; 47:  DATAPATH
                DCD     DP0_IRQHandler            ; 48:  DATAPATH int0
                DCD     DP1_IRQHandler            ; 49:  DATAPATH int1
                DCD     DP2_IRQHandler            ; 50:  DATAPATH int2
                DCD     BOD_OUT_IRQHandler        ; 51:  BOD_OUT
                DCD     BLE_LL_IRQHandler         ; 52:  BLE low level stack
__Vectors_End

__Vectors_Size  EQU     __Vectors_End - __Vectors

                AREA    |.text|, CODE, READONLY


; Reset Handler

Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                LDR     R1, =SYS_MODE_REG
                LDR     R0, [R1]
                LDR     R2, =REMAP_BIT
                ORR     R0, R0, R2
                STR     R0, [R1]

                LDR     R0, =__initial_sp
                MSR     MSP, R0

                IMPORT  SystemInit
                IMPORT  __main
                LDR     R0, =SystemInit
                BLX     R0
                LDR     R0, =__main
                BX      R0
                ENDP
                ALIGN

                AREA    in_ram, CODE
enter_sleep_in_ram\
                PROC
                EXPORT  enter_sleep_in_ram
#if defined(CFG_SW_RETEN)
                EXPORT  respawn_entry
                IMPORT  msp_addr
                IMPORT  psp_addr
                PUSH    {LR, R4-R11}
                LDR     R0, =msp_addr
                MRS     R1, MSP
                STR     R1, [R0]
    #if defined(CFG_USE_RTOS)
                LDR     R0, =psp_addr
                MRS     R1, PSP
                STR     R1, [R0]
    #endif
                WFI
respawn_entry   LDR     R1, =SYS_MODE_REG
                LDR     R0, [R1]
                LDR     R2, =REMAP_BIT
                ORR     R0, R0, R2
                STR     R0, [R1]

                LDR     R0, =msp_addr
                LDR     R1, [R0]
                MSR     MSP, R1
    #if defined(CFG_USE_RTOS)
                LDR     R0, =psp_addr
                LDR     R1, [R0]
                MSR     PSP, R1
                MOV     R0, #0x2
                MSR     CONTROL, R0
    #endif
                POP     {LR, R4-R11}
#else
                WFI
#endif
                ; delay for some time, to make sure flash is ready
                MOV     R1, #0
                MOV     R0, #0x45
                B       loop_start
loop_core       ISB
                ADD     R1, R1, #1
loop_start      CMP     R1, R0
                BCC     loop_core
                BX      LR
                ENDP
                ALIGN

; Dummy Exception Handlers (infinite loops which can be modified)
                AREA    |.text|, CODE, READONLY

NMI_Handler     PROC
                EXPORT  NMI_Handler               [WEAK]
                B       .
                ENDP
HardFault_Handler\
                PROC
                EXPORT  HardFault_Handler         [WEAK]
                B       .
                ENDP
MemManage_Handler\
                PROC
                EXPORT  MemManage_Handler         [WEAK]
                B       .
                ENDP
BusFault_Handler\
                PROC
                EXPORT  BusFault_Handler          [WEAK]
                B       .
                ENDP
UsageFault_Handler\
                PROC
                EXPORT  UsageFault_Handler        [WEAK]
                B       .
                ENDP
SVC_Handler     PROC
                EXPORT  SVC_Handler               [WEAK]
                B       .
                ENDP
DebugMon_Handler\
                PROC
                EXPORT  DebugMon_Handler          [WEAK]
                B       .
                ENDP
PendSV_Handler\
                PROC
                EXPORT  PendSV_Handler            [WEAK]
                B       .
                ENDP
SysTick_Handler\
                PROC
                EXPORT  SysTick_Handler           [WEAK]
                B       .
                ENDP

Default_Handler PROC
; ToDo:  Add here the export definition for the device specific external interrupts handler
                EXPORT  DIG_WAKEUP_IRQHandler     [WEAK]
                EXPORT  OSC_EN_IRQHandler         [WEAK]
                EXPORT  ACMP0_IRQHandler          [WEAK]
                EXPORT  ACMP1_IRQHandler          [WEAK]
                EXPORT  CHARGER_IRQHandler        [WEAK]
                EXPORT  RTC_IRQHandler            [WEAK]
                EXPORT  RTC_NEW_IRQHandler        [WEAK]
                EXPORT  TSC_WAKEUP_IRQHandler     [WEAK]
                EXPORT  TSC_IRQHandler            [WEAK]
                EXPORT  GPIOA_IRQHandler          [WEAK]
                EXPORT  GPIOB_IRQHandler          [WEAK]
                EXPORT  DMA_IRQHandler            [WEAK]
                EXPORT  PIN_INTR0_IRQHandler      [WEAK]
                EXPORT  PIN_INTR1_IRQHandler      [WEAK]
                EXPORT  PIN_INTR2_IRQHandler      [WEAK]
                EXPORT  PIN_INTR3_IRQHandler      [WEAK]
                EXPORT  OSC_INT_LOW_IRQHandler    [WEAK]
                EXPORT  USB_IRQHandler            [WEAK]
                EXPORT  FLEXCOMM0_IRQHandler      [WEAK]
                EXPORT  FLEXCOMM1_IRQHandler      [WEAK]
                EXPORT  FLEXCOMM2_IRQHandler      [WEAK]
                EXPORT  FLEXCOMM3_IRQHandler      [WEAK]
                EXPORT  BLE_IRQHandler            [WEAK]
                EXPORT  FSP_IRQHandler            [WEAK]
                EXPORT  QDEC0_IRQHandler          [WEAK]
                EXPORT  QDEC1_IRQHandler          [WEAK]
                EXPORT  TIMER0_IRQHandler         [WEAK]
                EXPORT  TIMER1_IRQHandler         [WEAK]
                EXPORT  TIMER2_IRQHandler         [WEAK]
                EXPORT  TIMER3_IRQHandler         [WEAK]
                EXPORT  WDT_IRQHandler            [WEAK]
                EXPORT  ADC_IRQHandler            [WEAK]
                EXPORT  DAC_IRQHandler            [WEAK]
                EXPORT  XTAL_IRQHandler           [WEAK]
                EXPORT  FLASH_IRQHandler          [WEAK]
                EXPORT  QUAD_SPI_IRQHandler       [WEAK]
                EXPORT  PWM_IRQHandler            [WEAK]
                EXPORT  RTC_CAP_IRQHandler        [WEAK]
                EXPORT  RNG_IRQHandler            [WEAK]
                EXPORT  CHRG_I2C_IRQHandler       [WEAK]
                EXPORT  CALIB_IRQHandler          [WEAK]
                EXPORT  USB_FIQ_IRQHandler        [WEAK]
                EXPORT  BLE_TX_IRQHandler         [WEAK]
                EXPORT  BLE_RX_IRQHandler         [WEAK]
                EXPORT  BLE_FREQ_HOP_IRQHandler   [WEAK]
                EXPORT  PROP_TX_IRQHandler        [WEAK]
                EXPORT  PROP_RX_IRQHandler        [WEAK]
                EXPORT  DP_IRQHandler             [WEAK]
                EXPORT  DP0_IRQHandler            [WEAK]
                EXPORT  DP1_IRQHandler            [WEAK]
                EXPORT  DP2_IRQHandler            [WEAK]
                EXPORT  BOD_OUT_IRQHandler        [WEAK]
                EXPORT  BLE_LL_IRQHandler         [WEAK]

; ToDo:  Add here the names for the device specific external interrupts handler
DIG_WAKEUP_IRQHandler
OSC_EN_IRQHandler
ACMP0_IRQHandler
ACMP1_IRQHandler
CHARGER_IRQHandler
RTC_IRQHandler
RTC_NEW_IRQHandler
TSC_WAKEUP_IRQHandler
TSC_IRQHandler
GPIOA_IRQHandler
GPIOB_IRQHandler
DMA_IRQHandler
PIN_INTR0_IRQHandler
PIN_INTR1_IRQHandler
PIN_INTR2_IRQHandler
PIN_INTR3_IRQHandler
OSC_INT_LOW_IRQHandler
USB_IRQHandler
FLEXCOMM0_IRQHandler
FLEXCOMM1_IRQHandler
FLEXCOMM2_IRQHandler
FLEXCOMM3_IRQHandler
BLE_IRQHandler
FSP_IRQHandler
QDEC0_IRQHandler
QDEC1_IRQHandler
TIMER0_IRQHandler
TIMER1_IRQHandler
TIMER2_IRQHandler
TIMER3_IRQHandler
WDT_IRQHandler
ADC_IRQHandler
DAC_IRQHandler
XTAL_IRQHandler
FLASH_IRQHandler
QUAD_SPI_IRQHandler
PWM_IRQHandler
RTC_CAP_IRQHandler
RNG_IRQHandler
CHRG_I2C_IRQHandler
CALIB_IRQHandler
USB_FIQ_IRQHandler
BLE_TX_IRQHandler
BLE_RX_IRQHandler
BLE_FREQ_HOP_IRQHandler
PROP_TX_IRQHandler
PROP_RX_IRQHandler
DP_IRQHandler
DP0_IRQHandler
DP1_IRQHandler
DP2_IRQHandler
BOD_OUT_IRQHandler
BLE_LL_IRQHandler

                B       .
                ENDP

                ALIGN


; User Initial Stack & Heap

                IF      :DEF:__MICROLIB

                EXPORT  __initial_sp
                EXPORT  __heap_base
                EXPORT  __heap_limit

                ELSE

                IMPORT  __use_two_region_memory
                EXPORT  __user_initial_stackheap

__user_initial_stackheap PROC
                LDR     R0, =  Heap_Mem
                LDR     R1, =(Stack_Mem + Stack_Size)
                LDR     R2, = (Heap_Mem +  Heap_Size)
                LDR     R3, = Stack_Mem
                BX      LR
                ENDP

                ALIGN

                ENDIF


                END
